from flask import session, request, Response, url_for, redirect
from typing import List
from functools import wraps
from api.utils.errors import AuthenticationError
import logging

def roles_allowed(roles: List[str]):
    """Decorator that only allows access to a route if the user is logged in and has at least
    one of the specified roles"""
    def decorator(func):
        @wraps(func)
        def wrapper(*args, **kwargs):
            user = session.get('user')
            logging.debug("Secured page requested.")
            if not user:
                logging.warning(f"Unauthorized request from: {request.remote_addr}")
                return Response("Not authorized", 401)
            if any(role in user['roles'] for role in roles):
                # User has a required role, route request back to the original handler
                logging.debug(f"Access granted to user: {user['email']}")
                return func(*args, **kwargs)
            else:
                logging.warning(f"Forbidden request from: {request.remote_addr}")
                return Response("Not authorized", 403)

        return wrapper
    return decorator



def prompt_login(func):
    """Decorator that redirects the user to a login page if they are not logged in"""
    @wraps(func)
    def wrapper(*args, **kwargs):
        user = session.get('user')

        if user:
            # User is logged in, route request back to the original handler
            return func(*args, **kwargs)

        return redirect(url_for('login'))

    return wrapper