import logging
from flask import render_template, request, jsonify, redirect, session
from flask.helpers import make_response, url_for
from flask.wrappers import Response
from api.services import datastore
from api.services.auth import parse_user_from_token, redirect_to_auth_login
from api.decorators.auth import roles_allowed, prompt_login
from api.utils.errors import ValidationError, AuthenticationError
from main import app, limiter

@app.route('/', methods=['GET'])
@limiter.limit("100 per minute")
def home():
    logging.debug("Front page requested.")
    csrfToken = session.get('csrf')
    return render_template('mymap.html', csrfToken=csrfToken)


#Route for storing new hive locations
@app.route('/api/hives', methods=['POST'])
def insert_new():
    logging.debug("Posting new hive")

    try:
        result = datastore.store_location(request.get_json())
        return make_response(jsonify(result), 200)
    except ValidationError as e:
        return make_response(jsonify({"error": e.message}), 400)
    except Exception as e:
        #If something went wrong, come here, print error and send status 400
        logging.error(e)
        return make_response(jsonify({"error":e.__class__.__name__}), 400)


#Route for storing new hive locations
@app.route('/api/hives', methods=['GET'])
@limiter.limit("100 per minute")
def get_locations():
    logging.debug("Getting all hives")
    try:
        result = datastore.get_locations()
        return make_response(jsonify(result), 200)
    except Exception as e:
        #If something went wrong, come here, print error and send status 400
        logging.error(e)
        return make_response(jsonify({"error":e.__class__.__name__}), 400)


#Route for editing a hive location
@app.route('/api/hives', methods=['PUT'])
@roles_allowed(roles=['admin'])
@limiter.exempt
def edit_location():
    logging.debug("Editing a hive")
    try:
        result = datastore.edit_location(request.get_json())
        return make_response(jsonify(result), 200)
    except ValidationError as e:
        return make_response(jsonify({"error": e.message}), 400)
    except Exception as e:
        #If something went wrong, come here, print error and send status 400
        logging.error(e)
        return make_response(jsonify({"error":e.__class__.__name__}), 400)


#Route for deleting a hive location
@app.route('/api/hives', methods=['DELETE'])
@roles_allowed(roles=['admin'])
@limiter.exempt
def delete_location():
    logging.debug("Deleting a hive")
    try:
        result = datastore.delete_location(request.get_json())
        return make_response(jsonify({"deleted":result}, 204))
    except Exception as e:
        #If something went wrong, come here, print error and send status 400
        logging.error(e)
        return make_response(jsonify({"error":e.__class__.__name__}), 400)


# This will contain the actual admin page
@app.route('/admin', methods=['GET'])
@prompt_login
@roles_allowed(roles=['admin'])
def admin_dashboard():
    user = session.get('user')
    csrfToken = session.get('csrf')
    logging.info(f"Admin {user['email']} logged in")
    return render_template('admin.html', email=user['email'], csrfToken=csrfToken)


# This will work as the login endpoint
@app.route('/login', methods=['GET'])
def login():
    logging.debug("Login page requested.")

    # Generate the url where OAuth provider will send the auth token
    callback_url = url_for('login_callback', _external=True)
    return redirect_to_auth_login(callback_url=callback_url)


# This will work as the login endpoint
@app.route('/logout', methods=['GET'])
def logout():
    logging.debug("Logout requested.")

    session.pop('user')

    return redirect(url_for('home'))


# This will be used as the redirect address for OAuth
@app.route('/api/login-callback', methods=['GET'])
def login_callback():
    logging.debug("Received callback from OAuth server")
    try:
        # Parses the user id and roles from the response. Authlib
        # grabs the token from the response context that does not have
        # to be passed explicitly.
        user = parse_user_from_token()
        logging.info(f"Received token for: {user['email']}")

        # Stores the user information in the session context. This will cause
        # flask to automatically return a session cookie that will be used to 
        # id the user during further requests.
        session['user'] = user
        
        if 'admin' in user['roles']:
            return redirect(url_for("admin_dashboard"))

    except Exception as e:
        logging.error(f"Exception during authentication:")
        logging.error(e)
    
    return redirect(url_for('home'))

