from flask.globals import request
from flask.helpers import make_response
from google.cloud import datastore
from flask import jsonify, Markup
import os
import time
import logging
from api.utils.validate import ValidationError, validate

HIVE_KIND = 'HiveLocation'

# Method for storing a new hive location
def store_location(data):
    
    # Try validating the entity
    try:
        sanitated_data = _sanitate_data(data)
        validate(sanitated_data)
    except ValidationError:
        raise

    #intantiate a new datastore client
    datastore_client = _get_datastore_client()
    kind = HIVE_KIND
    timestamp = time.strftime('%Y-%m-%d-%H:%M:%S')

    #set name and kind for key
    key = datastore_client.key(kind)

    #Create datastore entity and set key
    datastore_entity = datastore.Entity(key)

    #Update hive data to entity
    datastore_entity.update({
        'name': sanitated_data["name"],
        'timestamp': timestamp,
        'lat': sanitated_data["lat"],
        'lng': sanitated_data["lng"],
        'person': sanitated_data["person"],
        'keeper': sanitated_data["keeper"],
        'email': sanitated_data["email"]
    })

    #Store the info to datastore
    datastore_client.put(datastore_entity)
    
    updated_entity = {
        'name': datastore_entity["name"],
        'lat': datastore_entity["lat"],
        'lng': datastore_entity["lng"],
        'timestamp': datastore_entity["timestamp"],
        'kind': datastore_entity.kind,
        'id': datastore_entity.id,
        'person': datastore_entity["person"],
        'keeper': datastore_entity["keeper"],
        'email': datastore_entity["email"]
    }

    return updated_entity


# Method for getting all hive locations
def get_locations():
    #intantiate a new datastore client
    datastore_client = _get_datastore_client()
    locations = []
    for entry in datastore_client.query(kind='HiveLocation').fetch():
        locations.append({
            "name": entry['name'],
            "lat": entry['lat'],
            "lng": entry['lng'],
            'timestamp': entry['timestamp'],
            'id': entry.id
        })

    return locations



# Method for deleting a suggested hive location
def delete_location(data):
    #intantiate a new datastore client
    datastore_client = _get_datastore_client()

    kind = HIVE_KIND
    id = data['id']
    
    entity_key = datastore_client.key(kind, int(id))

    datastore_client.delete(entity_key)

    return id



# Method for editing a hive location
def edit_location(data):
    #intantiate a new datastore client
    datastore_client = _get_datastore_client()
    kind = HIVE_KIND

    id = data['id']

    entity_key = datastore_client.key(kind, int(id))

    entity = datastore_client.get(entity_key)

    for key in data:
        if key in entity:
            entity[key] = data[key]

    # Try validating the updated entity
    try:
        validate(entity)
    except ValidationError:
        raise

    datastore_client.put(entity)

    return entity


def _sanitate_data(data: dict):
    data_cp = data.copy()

    for key, value in data_cp.items():
        if type(value) is str:
            data_cp[key] = Markup.striptags(value)

    return data_cp


def get_user(email):
    client = _get_datastore_client(default_namespace=True)

    try:
        # Fetches the secret from the database
        query = client.query(kind='UserAccount')
        # Filters the query to only include results with matching name
        query.add_filter('email','=',email)
        # Packages the query response in an iterator
        query_iter = iter(query.fetch())
        # Grabs the first filtered entity or None
        entity = next(query_iter, None)
        print(entity)
        return entity
    except Exception as e:
        logging.error(e)
        return None


# def create_test_user(email, roles, client):
#     kind = 'UserAccount'

#     #set name and kind for key
#     key = client.key(kind)

#     #Create datastore entity and set key
#     datastore_entity = datastore.Entity(key)

#     #Update hive data to entity
#     datastore_entity.update({
#         'email': email,
#         'roles': roles
#     })

#     #Store the info to datastore
#     client.put(datastore_entity)



def _get_datastore_client(default_namespace: bool = False):
    """Instantiates a new datastore client. The client's namespace is set to match the value
    stored in the environment variable 'DEPLOYMENT_ENVIRONMENT" or defalut if the
    variable is unset"""

    # If running in app engine
    if os.getenv('GAE_ENV', '').startswith('standard'):
        if default_namespace:
            client = datastore.Client(namespace=None)
            logging.info(f"Instantiating datastore client with default namespace")
        else:
            namespace=os.getenv('GAE_SERVICE')
            client = datastore.Client(namespace=namespace)
            logging.info(f"Instantiating datastore client with namespace: {namespace}")
    else:
        # If running on local machine or testing environment
        os.environ["DATASTORE_PROJECT_ID"] = "emulated-project"

        from google.auth.credentials import AnonymousCredentials

        client = datastore.Client(
            project="emulated-project",
            namespace='ns_test',
            credentials=None
        )

    return client


def get_secret(name: str):
    """Function for getting "secrets" (API keys, client id's) from the project datastore.
    In a production app this should be changed to fetch the secrets from a secret manager
    instead."""
    # TODO poikkeuskäsittelyä jne
    print("Trying to fetch a SECRET:", name)
    client = _get_datastore_client(default_namespace=True)

    try:
        # Fetches the secret from the database
        query = client.query(kind='Secret')
        # Filters the query to only include results with matching name
        query.add_filter('Name','=',name)
        # Packages the query response in an iterator
        query_iter = iter(query.fetch())
        # Grabs the first filtered entity or None
        entity = next(query_iter, None)
        return entity['Value']
    except Exception as e:
        logging.error(e)
        return None