from flask import Flask
import os
import logging
import google.cloud.logging
from google.cloud.logging.handlers import CloudLoggingHandler, setup_logging

from opencensus.common.transports.async_ import AsyncTransport
from opencensus.ext.stackdriver.trace_exporter import StackdriverExporter
from opencensus.trace.tracer import Tracer

from opencensus.ext.flask.flask_middleware import FlaskMiddleware
from opencensus.trace.samplers import ProbabilitySampler
from flask_babel import Babel
from flask_limiter import Limiter
from flask_limiter.util import get_remote_address
from api.services.csrf import checkCsrfToken

app = Flask(__name__)


# TODO move all configuration to separate module / files
if os.getenv('GAE_ENV', '').startswith('standard'):
    client = google.cloud.logging.Client()
    handler = CloudLoggingHandler(client)
    setup_logging(handler)

logging.getLogger().setLevel(logging.DEBUG)

if os.getenv('GAE_ENV', '').startswith('standard'):
    logging.info("Loading app configuration from file")
    app.config.from_pyfile('config/config.py')

if not app.secret_key:
    logging.info("Generating a random secret key")
    app.secret_key = os.urandom(16)

# Add rate limiter with default limits
limiter = Limiter(
    app,
    key_func=get_remote_address,
    default_limits=["200 per day", "50 per hour", "10 per minute"],
)

# Initialize babel
babel = Babel(app)
@babel.localeselector
def get_locale():
    return request.accept_languages.best_match(['fi', 'en'])

def main():
    sde = StackdriverExporter(
        project_id=os.environ.get("GCP_PRJECT_ID"),
        transport=AsyncTransport)
    FlaskMiddleware(
        app,
        exporter=sde,
        sampler=ProbabilitySampler(rate=1.0)
    )

app.before_request(checkCsrfToken)

from api.routes import *
# app.config.from_object("config.DevConfig")

if __name__ == '__main__':
    host = os.getenv("HOST", "127.0.0.1")
    port = os.getenv("PORT", "5000")
    main()

    logging.info(f"Starting buzzing service at {host}:{port}")
    app.run(host=host, port=port)
    logging.info(f"Stopping buzzing service at {host}:{port}")
