import {deleteLocation, editLocation, loadHiveLocations} from './data.js';

/**
 * Fetches all previously added hive location suggestions
 * and populates a table with the received data.
 */
async function createTable() {
  try {
    const response = await loadHiveLocations();
    const div = document.getElementById('hives');
    const oldTable = document.getElementById('hiveTable');

    // Removes the old table so only up-to-date data will be shown
    if (oldTable) {
      div.removeChild(oldTable);
    }

    // Creates a new table
    const table = document.createElement('table');
    table.id = 'hiveTable';
    div.appendChild(table);

    // Populates the table if any suggestions have been made
    if (response.length > 0) {
      /*
      Cell headings are fetched from the first suggestion,
      so adding new data fields will not require changes here
      */
      const headingRow = createHeadingRow(response[0]);
      table.appendChild(headingRow);

      // Creates a row per each suggestion
      response.forEach((suggestion) => {
        const row = createContentRow(suggestion);
        table.appendChild(row);
      });
    }
  } catch (e) {
    console.log('error:', e);
  }
}

/**
 * Creates the heading row based on suggestion keys
 * @param {*} content
 * @return {*} heading row
 */
function createHeadingRow(content) {
  const tr = document.createElement('tr');

  // Creates a heading cell per each key on suggestion data
  Object.keys(content).map((key) => {
    const th = document.createElement('th');
    th.textContent = key;
    tr.appendChild(th);
  });

  return tr;
}

/**
 * Creates the heading row based on suggestion values
 * @param {*} content
 * @return {*} content row
 */
function createContentRow(content) {
  const id = content['id'];
  const tr = document.createElement('tr');

  // Creates a data cell per each value on suggestion data
  Object.entries(content).map(([key, value]) => {
    const td = document.createElement('td');
    td.textContent = value;

    // Element id's will store the corresponding data field name and entity id
    td.id = `${key}_${id}`;

    // All data except id's and timestamps will be editable
    if (key !== 'id' && key !== 'timestamp') {
      td.addEventListener('click', () => activateEditMode(td));
    }
    tr.appendChild(td);
  });

  /*
  Creates a button to save edited data.
  Does not check if the data has actually been altered
  */
  const tdSave = document.createElement('td');
  const saveButton = document.createElement('button');
  saveButton.textContent = 'Save';
  saveButton.addEventListener('click', () => saveContent(tr));
  tdSave.appendChild(saveButton);
  tr.appendChild(tdSave);

  // Creates a button to delete the entity.
  const tdDel = document.createElement('td');
  const deleteButton = document.createElement('button');
  deleteButton.textContent = 'Delete';
  deleteButton.addEventListener('click', () => deleteContent(tr));
  tdDel.appendChild(deleteButton);
  tr.appendChild(tdDel);

  return tr;
}

/**
 * Changes the child text node
 * to an input field with the value set to the original text.
 * Registers a 'focusout' event listener to change the field back to text node.
 * @param {*} parent
 */
function activateEditMode(parent) {
  // If already editing, return
  if (parent.childElementCount > 0) {
    return;
  }

  // Creates an input element and sets the value to match the original text
  const input = document.createElement('input');
  input.type = 'text';
  input.value = parent.textContent;

  // Registers an event listener to deactivate editing
  input.addEventListener('focusout', () => deactivateEditMode(input, parent));

  // Removes the text data from the parent
  parent.textContent = '';
  parent.appendChild(input);

  /*
  Gives automatic focus to the input element
  so the event listener will react immediately when focusing elsewhere
  */
  input.focus();
}

/**
 * Removes the input element and sets the parent's text content
 * to match the input field's value
 * @param {*} input
 * @param {*} parent
 */
function deactivateEditMode(input, parent) {
  const text = input.value;
  parent.removeChild(input);
  parent.textContent = text;
}

/**
 * Packages the edited entity data into an object
 * and calls to create a put request to update the data
 * @param {*} tr
 */
async function saveContent(tr) {
  const entity = {};

  /*
  Iterates through every cell on the row,
  and appends the data to the entity object
  with the key parsed from the element's id attribute
  and value from the cell's text content
  */
  for (const node of tr.childNodes) {
    const idString = node.id;
    if (!node.id) continue;
    const splitpoint = idString.lastIndexOf('_');
    const name = idString.substring(0, splitpoint);
    entity[name] = node.textContent;
  }

  await editLocation(entity);
  createTable();
}

/**
 * Packages the entity data into an object
 * and calls to create a delete request
 * @param {*} tr
 */
async function deleteContent(tr) {
  const entity = {};

  /*
  Iterates through every cell on the row,
  and appends the data to the entity object
  with the key parsed from the element's id attribute
  and value from the cell's text content
  */
  for (const node of tr.childNodes) {
    const idString = node.id;
    if (!node.id) continue;
    const splitpoint = idString.lastIndexOf('_');
    const name = idString.substring(0, splitpoint);
    entity[name] = node.textContent;
  }

  await deleteLocation(entity);
  createTable();
}

window.onload = function() {
  createTable();
};
