import {saveLocation, loadHiveLocations} from './data.js';

let selectionMarker;
let map;
const allSuggestions = [];
const TWITTER_URL =
  'https://twitter.com/intent/tweet?text=I%20found%20a%20new%20beehive!%20Check%20out%20Beeings!&url=https://beeings.org/';
const TWITTER_ICON = 'static/icons/tweet-button.png';
const EMAIL_URL =
  'mailto:?subject=I found a new beehive!&amp;body=Check out Beeings! https://beeings.org/';
const EMAIL_ICON = 'static/icons/email-button.png';

/**
 * A green marker for new suggestions.
 * Does not include the high-res "retina" icon at the moment.
 */
const GreenIcon = L.Icon.extend({
  options: {
    iconUrl: 'static/icons/marker-icon-green.png',
    iconAnchor: [12, 41],
    iconSize: [25, 41],
    shadowSize: [41, 41],
    shadowUrl: 'static/icons/marker-shadow.png',
    popupAnchor: [0, -28],
  },
});

/**
 * Adds a green marker to mark a place for a possible suggestion.
 * @param {*} map
 * @param {*} latlng
 */
function addSuggestionMarker(map, latlng) {
  if (!selectionMarker) {
    const greenIcon = new GreenIcon();
    selectionMarker = L.marker(latlng, {icon: greenIcon}).addTo(map);
  }
  selectionMarker.setLatLng(latlng);
  console.log('Marker moved to:', latlng);
  openSuggestionPopup();
}

/**
 * Opens a pop-up window to let the user suggest a new location for a beehive.
 * Bound to click events on the leaflet map.
 * Pop-up content is created in createSuggestionForm().
 */
function openSuggestionPopup() {
  console.log('Opening popup');
  const popup = L.popup().setContent(createSuggestionForm());
  selectionMarker.bindPopup(popup).openPopup();

  // Remove the marker when the popup is closed from the cross symbol
  selectionMarker.addEventListener('popupclose', (e) => {
    console.log('Popup closed');
    selectionMarker.remove();
    selectionMarker = undefined;
  });
}

/**
 * Creates a form that can be used to submit suggestions
 * from inside the popup.
 * @return {*} suggestion form
 */
function createSuggestionForm() {
  const form = document.createElement('form');
  const nameLabel = document.createElement('label');
  nameLabel.textContent = 'Suggest a new location:';
  const nameInput = document.createElement('input');
  nameInput.setAttribute('required', '');
  nameInput.setAttribute('placeholder', 'e.g. Beehive');
  nameLabel.appendChild(nameInput);

  const personLabel = document.createElement('label');
  personLabel.textContent = 'Suggester\'s full name:';
  const personInput = document.createElement('input');
  personInput.setAttribute('required', '');
  personInput.setAttribute('placeholder', 'e.g. John Doe');
  personLabel.appendChild(personInput);

  const emailLabel = document.createElement('label');
  emailLabel.textContent = 'Suggester\'s email:';
  const emailInput = document.createElement('input');
  emailInput.setAttribute('required', '');
  emailInput.setAttribute('placeholder', 'e.g. hello@email.com');
  emailLabel.appendChild(emailInput);

  const keeperLabel = document.createElement('label');
  keeperLabel.textContent = 'Beekeeper or sponsor:';
  const keeperInput = document.createElement('input');
  keeperInput.setAttribute('required', '');
  keeperInput.setAttribute('placeholder', 'e.g. John Doe');
  keeperLabel.appendChild(keeperInput);

  const button = document.createElement('button');
  button.setAttribute('class', 'saveButton');
  button.textContent = 'Save';

  form.appendChild(nameLabel);
  form.appendChild(personLabel);
  form.appendChild(emailLabel);
  form.appendChild(keeperLabel);
  form.appendChild(button);
  form.onsubmit = (e) => {
    e.preventDefault();
    submitSuggestion(
        nameInput.value,
        personInput.value,
        keeperInput.value,
        emailInput.value,
    );
  };

  return form;
}

/**
 * Creates a link with an image
 * @param  {string} link page to redirect
 * @param  {string} image button icon
 * @return {*} shareButton
 */
function createShareButton(link, image) {
  const shareButton = document.createElement('a');
  shareButton.setAttribute('href', link);
  shareButton.setAttribute('target', '_blank');
  shareButton.innerHTML = `<img src="${image}" class="share">`;
  return shareButton;
}

/**
 * Submits the suggested location to server. Updates popup content
 * depending on the result. Saves a new marker if addition successful.
 * @param {*} name
 * @param {*} person
 * @param {*} keeper
 * @param {*} email
 */
async function submitSuggestion(name, person, keeper, email) {
  // Get location data from the selection markers current position
  const {lat, lng} = selectionMarker.getLatLng();

  const data = {
    name: name,
    lat: lat,
    lng: lng,
    person: person,
    keeper: keeper,
    email: email,
  };
  const popup = selectionMarker.getPopup();

  try {
    popup.setContent(updatePopupContent('Saving the suggested place...'));
    const responseJson = await saveLocation(data);
    const suggestion = responseJson;
    const addedMarker = saveSuggestion(lat, lng, suggestion);

    popup.setContent(updatePopupContent('Suggestion saved succesfully!'));
    addedMarker.bindPopup(popup).openPopup();

    setTimeout(() => {
      addedMarker.closePopup();
      popup.setContent(updatePopupName(suggestion.name));
    }, 2000);
  } catch (error) {
    popup.setContent(updatePopupContent('Error, please try again!'));
    console.log(error);
  }
}

/**
 * Saves a new suggestion. This function is called after the server
 * responds with a success
 * @param {*} lat latitude
 * @param {*} lng longitude
 * @param {*} suggestion name
 * @return {*} added marker
 */
function saveSuggestion(lat, lng, suggestion) {
  console.log(suggestion);
  /*
  Saves the suggestion for further use,
  the created marker is appended to the suggestion object
  */
  const addedMarker = addMarker(lat, lng);
  suggestion.marker = addedMarker;
  allSuggestions.push(suggestion);

  // Remove the temporary marker
  selectionMarker.remove();
  selectionMarker = undefined;

  return addedMarker;
}

/**
 * Updates popup
 * @param {*} text
 * @return {*} created popup
 */
function updatePopupContent(text) {
  const p = document.createElement('p');
  p.textContent = text;
  return p;
}

/**
 * Updates popup
 * @param {*} text
 * @return {*} created popup
 */
function updatePopupName(text) {
  const div = document.createElement('div');
  const item = document.createElement('h2');
  item.setAttribute('class', 'itemTitle');
  item.textContent = text;
  const shareButtons = document.createElement('div');
  const twitterShareButton = createShareButton(TWITTER_URL, TWITTER_ICON);
  shareButtons.appendChild(twitterShareButton);
  const emailShareButton = createShareButton(EMAIL_URL, EMAIL_ICON);
  shareButtons.appendChild(emailShareButton);
  div.appendChild(item);
  div.appendChild(shareButtons);
  return div;
}

/**
 * Loads data from previously submitted suggestions and populates
 * the map with markers pointing to their locations
 */
async function initializeMarkers() {
  try {
    const response = await loadHiveLocations();
    console.log(response);

    response.forEach((suggestion) => {
      allSuggestions.push(suggestion);
      const lat = suggestion.lat;
      const lng = suggestion.lng;
      const marker = addMarker(lat, lng);
      const popup = L.popup().setContent(updatePopupName(suggestion.name));
      marker.bindPopup(popup);
    });
  } catch (e) {
    console.log('error:', e);
  }
}

/**
 * Adds a marker to the map
 * @param {*} lat
 * @param {*} lng
 * @return {*} marker
 */
function addMarker(lat, lng) {
  const marker = L.marker({lat, lng}).addTo(map);
  return marker;
}

/** Initializes map */
function initializeMap() {
  // initialize Leaflet
  map = L.map('map').setView({lon: 25.72088, lat: 62.24147}, 6);

  // add the OpenStreetMap tiles
  L.tileLayer('https://{s}.tile.openstreetmap.org/{z}/{x}/{y}.png', {
    maxZoom: 19,
    attribution:
      '&copy; <a href="https://openstreetmap.org/copyright">OpenStreetMap contributors</a>',
  }).addTo(map);

  // show the scale bar on the lower left corner
  L.control.scale().addTo(map);

  map.addEventListener('click', (e) => {
    addSuggestionMarker(map, e.latlng);
  });
}

export {initializeMap, initializeMarkers};
